from flask import Blueprint, render_template, redirect, url_for, request, jsonify
from models.message import Message
from models.author import Author
from sqlalchemy import or_, func, desc, case
from models.base import Session
from controllers.message_controller import MessageController # Corrected import path
import logging

logger = logging.getLogger(__name__)

bp = Blueprint('main', __name__)

@bp.route("/")
def index():
    return redirect(url_for('main.messages_page'))

@bp.route("/messages")
def messages_page():
    session = Session()
    try:
        total_messages = session.query(Message).count()
        return render_template('messages.html', total_messages=total_messages)
    finally:
        session.close()

@bp.route("/messages-json")
def messages_json():
    try:
        page = int(request.args.get('page', 1))
        result = MessageController.get_messages(page)
        
        if result is None:
            return jsonify({
                'error': 'Failed to load messages'
            }), 500
            
        return jsonify(result)  # Result already contains properly formatted data
        
    except Exception as e:
        logger.exception("Error in messages_json endpoint")
        return jsonify({
            'error': str(e)  # Convert error to string explicitly
        }), 500

@bp.route("/search-messages")
def search_messages():
    session = Session()
    try:
        query = request.args.get('q', '')
        page = int(request.args.get('page', 1))
        search_type = request.args.get('type', 'all')
        per_page = 10

        # Base query with join
        base_query = session.query(Message, Author).join(Author)

        if query:
            query_lower = query.lower()
            
            # Fix the case() syntax - pass conditions as individual arguments
            base_query = session.query(
                Message,
                Author,
                case(
                    (func.lower(Message.caption) == query_lower, 300),
                    (Message.caption.ilike(f'%{query_lower}%'), 200),
                    (Author.name.ilike(f'%{query_lower}%'), 100),
                    (Message.body.ilike(f'%{query_lower}%'), 50),
                    else_=0
                ).label('relevance')
            ).join(Author).filter(
                or_(
                    Message.caption.ilike(f'%{query_lower}%'),
                    Author.name.ilike(f'%{query_lower}%'),
                    Message.body.ilike(f'%{query_lower}%')
                )
            ).order_by(desc('relevance'), Message.timestamp.desc())

        # Calculate total results
        total_results = base_query.count()
        
        # Apply pagination
        results = base_query.offset((page - 1) * per_page).limit(per_page).all()

        # Group messages by author
        authors_dict = {}
        for msg, author, relevance in results:
            if author.id not in authors_dict:
                authors_dict[author.id] = {
                    'author_id': author.id,
                    'name': author.name,
                    'number': author.number,
                    'messages': [],
                    'latest_timestamp': msg.timestamp,
                    'message_count': 0
                }

            authors_dict[author.id]['messages'].append({
                'id': msg.id,
                'body': msg.body,
                'caption': msg.caption,
                'message_type': msg.message_type,
                'timestamp': msg.timestamp.isoformat() if msg.timestamp else None,
                'local_url': msg.local_url,
                'file_hash': msg.file_hash
            })
            authors_dict[author.id]['message_count'] += 1

        authors = list(authors_dict.values())
        authors.sort(key=lambda x: x['latest_timestamp'], reverse=True)

        total_pages = (total_results + per_page - 1) // per_page

        return jsonify({
            'authors': authors,
            'pagination': {
                'current_page': page,
                'total_pages': total_pages,
                'has_next': page < total_pages,
                'total_results': total_results
            }
        })

    except Exception as e:
        logger.exception("Search error occurred")
        return jsonify({
            'error': str(e),
            'authors': [],
            'pagination': {
                'current_page': 1,
                'total_pages': 0,
                'has_next': False,
                'total_results': 0
            }
        }), 500
    finally:
        session.close()

@bp.route("/edit-message", methods=['POST'])
def edit_message():
    session = Session()
    try:
        message_id = request.form.get('message_id')
        new_body = request.form.get('body')

        message = session.query(Message).filter(Message.id == message_id).first()
        if message:
            message.body = new_body
            session.commit()
            return jsonify({'status': 'success', 'message': 'Message updated successfully'})
        else:
            return jsonify({'status': 'error', 'message': 'Message not found'}), 404
    except Exception as e:
        logger.error(f"Error editing message: {str(e)}")
        session.rollback()
        return jsonify({'status': 'error', 'message': str(e)}), 500
    finally:
        session.close()

@bp.route("/delete-message", methods=['POST'])
def delete_message():
    session = Session()
    try:
        message_id = request.form.get('message_id')

        message = session.query(Message).filter(Message.id == message_id).first()
        if message:
            session.delete(message)
            session.commit()
            return jsonify({'status': 'success', 'message': 'Message deleted successfully'})
        else:
            return jsonify({'status': 'error', 'message': 'Message not found'}), 404
    except Exception as e:
        logger.error(f"Error deleting message: {str(e)}")
        session.rollback()
        return jsonify({'status': 'error', 'message': str(e)}), 500
    finally:
        session.close()

@bp.route("/messages-by-author")
def messages_by_author():
    page = int(request.args.get('page', 1))
    result = MessageController.get_messages_by_author(page)
    
    if result is None:
        return jsonify({'error': 'Error loading messages'}), 500
        
    return jsonify(result)  # This already contains the authors data structure

@bp.route("/about")
def about_page():
    return render_template('about.html')
